@@ -6,7 +6,7 @@
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation; either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -46,12 +46,21 @@
 
 #define SMALLBUF 256
 
+/* Christian <mail.kristian@yahoo.it> 
+ *
+ * This variable is global and referred from fdisk.c and cfdisk.c.
+ * Value of this variable is zero when extensions must be avaiable,
+ * othervise code must be compatible with linux fdisk and cfdisk. 
+ */
+
+int compat_mode = 1; /* Start compatible with fdisk and cfdisk */
+
 /* Here we store the struct with interface functions */
 static UICalls *uiquery;
 
 static int MEGABYTE_SECTORS (PedDevice* dev)
 {
-        return PED_MEGABYTE_SIZE / dev->sector_size;
+  return PED_MEGABYTE_SIZE / dev->sector_size;
 }
 
 
@@ -59,21 +68,24 @@ static int MEGABYTE_SECTORS (PedDevice* dev)
 /* TODO: Decide if these should be moved to UICalls */
 StrList *disk_type_list;
 StrList *fs_type_list;
-//StrList *fs_type_resize;
+/*StrList *fs_type_resize;*/
 StrList *fs_type_mkfs;
 StrList *flag_name_list;
 
 int
 init_flag_str ()
 {
-	PedPartitionFlag walk;
-	flag_name_list = NULL;
-	for (walk = ped_partition_flag_next(0); walk; walk =ped_partition_flag_next(walk)) {
-		flag_name_list = str_list_append(flag_name_list, ped_partition_flag_get_name(walk));
-		if (!flag_name_list)
-			return 0;
-	}
-	return 1;
+  PedPartitionFlag walk;
+  flag_name_list = NULL;
+  for (walk = ped_partition_flag_next(0); 
+       walk; walk =ped_partition_flag_next(walk)) 
+    {
+      flag_name_list = 
+	str_list_append(flag_name_list, ped_partition_flag_get_name(walk));
+      if (!flag_name_list)
+	return 0;
+    }
+  return 1;
 }
 
 int
@@ -210,6 +222,42 @@ _warn_ext_not_empty (PedPartition *part) {
 	return 1;
 }
 
+
+/* Get previous non-METADATA partition. */
+PedPartition *
+disk_get_prev_nmd_partition(PedDisk *disk, PedPartition *part) {
+	PedPartition *walk_a, *walk_b;
+	/* walk_a will iterate through partitions that are not METADATA, while
+	   walk_b will iterate through all partitions. When walk_b reaches our partition,
+	   walk_a is the partition we are looking for. */
+	walk_a = NULL;
+	walk_b = walk_a;
+	do {
+		if (ped_disk_next_partition(disk,walk_b) == part)
+			break;
+		walk_b = ped_disk_next_partition(disk,walk_b);
+		if (!walk_b || !(walk_b->type & PED_PARTITION_METADATA))
+			walk_a = walk_b;
+	} while (walk_b);
+	return walk_a;
+}
+
+/* This function returns the number of a metadata sectors, which are needed
+   after a partition, for example, in msdos partition table, the sector,
+   after a logical partition, describes the next logical partition */
+static PedSector metadata_tail_sectors(PedPartition *part) {
+	/* TODO: Make this support other partition types */
+	if (part == NULL || part->disk->type == NULL)
+		return 0;
+
+	if (!strcmp(part->disk->type->name, "msdos")
+	      && part->type & PED_PARTITION_LOGICAL) {
+		return 1;
+	}
+	return 0;
+}
+
+
 /* This function changes "sector" to "new_sector" if the new value lies
  * within the required range.
  */
@@ -384,13 +432,14 @@ snap_to_boundaries (PedGeometry* new_geom, PedGeometry* old_geom,
 /* This function makes a range less strict... */
 /* FIXME: Make this in a better way */
 static void
-fuzzify (PedGeometry *geom, PedDevice *dev, PedConstraint *constraint) {
+fuzzify (PedGeometry *geom, PedDevice *dev, PedConstraint *constraint,
+	 PedSector before, PedSector after) {
 	PedGeometry *new_geom;
 	if (!constraint)
 		constraint = ped_device_get_constraint(dev);
-	PedSector start = geom->start-127;
+	PedSector start = geom->start-before;
 	start = (start > 0 ? start : 0);
-	PedSector end = geom->start+127;
+	PedSector end = geom->start+after;
 	end = (end < dev->length-1LL ? end : dev->length-1LL);
 	ped_geometry_set (geom, start, end-start+1);
 	new_geom = ped_constraint_solve_nearest (constraint, geom);
@@ -400,7 +449,7 @@ fuzzify (PedGeometry *geom, PedDevice *dev, PedConstraint *constraint) {
 }
 /* This functions constructs a constraint from the following information:
  *      start, is_start_exact, end, is_end_exact.
- *      
+ *
  * If is_start_exact == 1, then the constraint requires start be as given in
  * "start".  Otherwise, the constraint does not set any requirements on the
  * start.
@@ -448,11 +497,11 @@ fix_part_position(PedDevice *dev, PedSector *start, PedSector *end,
 		if (*start != new_geom->start || *end != new_geom->end) {
 			*start = new_geom->start;
 			*end = new_geom->end;
-			
+
 		}
 		ped_geometry_destroy(new_geom);
 		return 1;
-	} 
+	}
 
 	return 0;
 
@@ -493,7 +542,7 @@ move_sector(PedDevice *dev, SectPos *value, PedSector shift) {
 		if (sector < 0LL)
 			sector = 0LL;
 	}
- 
+
 	if (end >= dev->length) {
 		end = dev->length - 1LL;
 		if (sector >= dev->length)
@@ -504,7 +553,7 @@ move_sector(PedDevice *dev, SectPos *value, PedSector shift) {
 		return 0;
 	value->sector = sector;
 	return 1;
-	
+
 }
 
 #define ESC	'\033'	/* ^[ */
@@ -515,7 +564,7 @@ int
 query_part_position(const char* prompt, const void* context, PartPos *pos,
                     PedSector first, PedSector last, PedDevice *dev,
                     PedConstraint *constraint, UIOpts *opts) {
-	
+
 	PedSector length = pos->end.sector - pos->start.sector + 1LL;
 	PedSector min_len = 0, max_len = dev->length;
 	PedGeometry *range = NULL;
@@ -528,14 +577,14 @@ query_part_position(const char* prompt, const void* context, PartPos *pos,
 
 	i = 0;
 	/* If we have a constraint, check whether we can move the partition start */
-	if (constraint && (constraint->start_align->grain_size == 0 || 
+	if (constraint && (constraint->start_align->grain_size == 0 ||
 	                   constraint->start_range->length == 1))
 		where = 's';
 	else {
 	/* If there the first possible sector is not the start of the selected partition,
 	we need one choice in the menu to make the start fixed and one to move it back.
 	When it is, 'b' and 's' do the same. We ditch 'b' for more friendly resize menu,
-	action_new should use 's' as it doesn't matter there. 
+	action_new should use 's' as it doesn't matter there.
 	FIXME: I wrote this fragment better, but ruined it. Now I'm too lazy to fix it */
 		if (first < pos->start.sector) possibilities[i++] = 'b';
 		possibilities[i++] = 's';
@@ -581,7 +630,7 @@ query_part_position(const char* prompt, const void* context, PartPos *pos,
 					if (def_str) ped_free(def_str);
 					return 0;
 				}
-	
+
 				done = 1;
 				/* Get the new size only if the user modified it */
 				if(strcmp(temp,def_str)) {
@@ -628,7 +677,7 @@ query_part_position(const char* prompt, const void* context, PartPos *pos,
 							last-63LL, 64LL);
 
 				pos->end.sector = last;
-				/* Desired: start = last-length+1LL */ 
+				/* Desired: start = last-length+1LL */
 				pos->start.sector = length;
 				pos->start.range = range;
 				move_sector(dev,&(pos->start),last-2*length+1LL);
@@ -644,7 +693,7 @@ query_part_position(const char* prompt, const void* context, PartPos *pos,
 		}
 	}
 	return 1;
-	
+
 }
 
 int
@@ -692,9 +741,9 @@ int
 get_partition (const char* prompt, PedDisk* disk, PedPartition** value)
 {
 	PedPartition*	part;
-	int		num; 
+	int		num;
 	char		buf[SMALLBUF];
-	if (!disk) 
+	if (!disk)
 		return 0;
 	if (!uiquery->getpart) {
 		snprintf(buf,SMALLBUF,_("%s number (1-%d)"),
@@ -745,7 +794,7 @@ get_disk_type (const char* prompt, const PedDiskType** value)
 		free (disk_type_name);
 		if (!disk_type)
 			return 0;
-		
+
 	} else {
 		if (!uiquery->getdisktype(prompt, &disk_type)) return 0;
 	}
@@ -783,7 +832,7 @@ _get_part_type (const char* prompt, const PedDisk* disk, PedPartitionType* type)
 	}
 
 	status = uiquery->getstring(prompt,&type_name,opts,locopts,-1);
-	
+
 	str_list_destroy (opts);
 	str_list_destroy (locopts);
 	if (!status) {
@@ -890,7 +939,7 @@ get_part_flag (const char* prompt, PedPartition *part, PedPartitionFlag *value)
 		return 0;
 	}
 	free (flag_name);
-	
+
 	*value = flag;
 	return 1;
 }
@@ -900,7 +949,7 @@ get_part_flag (const char* prompt, PedPartition *part, PedPartitionFlag *value)
    we want to query sector when UI_CUSTOM_VALUES is specified */
 typedef enum {
 	/* Query the user for a new sector */
-	GET_SECT = UI_CUSTOM_VALUES, 
+	GET_SECT = UI_CUSTOM_VALUES,
 	/* Initialise the range */
 	NO_RANGE = UI_CUSTOM_VALUES*2,
 	/* We are looking where to place the start of a new partition */
@@ -912,7 +961,7 @@ typedef enum {
 
 
 /* This function gets sector. The prev option specifies
-   the previous selected sector, used if the user 
+   the previous selected sector, used if the user
    wants to specify a size, not a position */
 static int
 get_sector (const char* prompt, PedDevice* dev, SectPos *value,
@@ -922,7 +971,7 @@ get_sector (const char* prompt, PedDevice* dev, SectPos *value,
 	char*	input = NULL;
 	char*	buf;
 	int	valid;
-	
+
 	/* If the user should be queried about the sector and a range needs to
 	   be initialised, initialise one with a length of 1 sector, but please,
 	   when using this function, NOTE that this is not always enough */
@@ -935,10 +984,11 @@ get_sector (const char* prompt, PedDevice* dev, SectPos *value,
 	}
 
 	def_str = ped_unit_format (dev, value->sector);
-	if (def_str) 
+	if (def_str)
 		input = strdup(def_str);
+	/* uiquery->getstring free input if it's not NULL */
 	uiquery->getstring(prompt,&input,NULL,NULL,1);
-	
+
 
 	/* def_str might have rounded sector a little bit.  If the user picked
 	 * the default, make sure the selected sector is identical to the
@@ -971,7 +1021,7 @@ get_sector (const char* prompt, PedDevice* dev, SectPos *value,
 	/* If the string starts with +, interpret as size */
 	if (prev >= 0 && input[0] == '+') {
 		/* TODO: Improve */
-		SectPos pos; 
+		SectPos pos;
 		/* We parse it into a SectPos */
 		buf = calloc(128,sizeof(char));
 		valid = sscanf(input, "+%127s", buf);
@@ -995,7 +1045,7 @@ get_sector (const char* prompt, PedDevice* dev, SectPos *value,
 		} else {
 			ped_geometry_destroy(pos.range);
 		}
-		
+
 	} else {
 		valid = ped_unit_parse(input, dev, &(value->sector),
 		                                   &(value->range));
@@ -1007,7 +1057,7 @@ get_sector (const char* prompt, PedDevice* dev, SectPos *value,
 /* Looks for the first suitable free space for a new partition */
 place_part_start(PedDisk *disk, PartPos *pos, PedPartitionType type) {
 	PedPartition *part = NULL;
-	type = (type & PED_PARTITION_LOGICAL) | PED_PARTITION_FREESPACE; 
+	type = (type & PED_PARTITION_LOGICAL) | PED_PARTITION_FREESPACE;
 	for (part = ped_disk_next_partition(disk, NULL);
 	     part; part = ped_disk_next_partition(disk,part)) {
 		if (part->type == type) {
@@ -1024,12 +1074,12 @@ place_part_start(PedDisk *disk, PartPos *pos, PedPartitionType type) {
 static int
 place_part_end(PedDisk *disk, PartPos *pos) {
 	PedPartition *part = ped_disk_get_partition_by_sector(disk,
-							pos->start.sector); 
+							pos->start.sector);
 	if (part->type & PED_PARTITION_FREESPACE) {
 		pos->end.sector = part->geom.end;
 		pos->end.range = ped_geometry_new (disk->dev,
 				 	part->geom.end - 7LL, 8LL);
-		return 1;	
+		return 1;
 	}
 	else
 		return 0;
@@ -1037,39 +1087,47 @@ place_part_end(PedDisk *disk, PartPos *pos) {
 
 
 /* Do not confuse this with query_part_position, this queries the user for
-   start and end of the partition and is called from the functions in here */ 
+   start and end of the partition and is called from the functions in here */
 static int
 get_position(PartPos* pos, PedDisk *disk, GSOpts opts)
 {
-
-	char *start;
-	char *end;
-	if (ped_unit_get_default() == PED_UNIT_CYLINDER) {
-		start = _("First cylinder");
-		end = _("Last cylinder or +size or +sizeMB or +sizeKB");
-	}
-	else if (ped_unit_get_default() == PED_UNIT_SECTOR) {
-		start = _("First sector");
-		end = _("Last sector or +size or +sizeMB or +sizeKB");
-	}
-	else {
-		start = _("Start");
-		end = _("End or +size");
-	}
-        if (!get_sector (start, disk->dev, &(pos->start), -1LL, opts))
-                return 0;
-
-	/* If we are creating a new partition and want to look where to
-           place its end ... */
-	if (opts & PLACE_END_NEWPART)
-		place_part_end(disk,pos);
-
-	/* We give the sector from the first query, so that the user can
-           specify size */
-        if (!get_sector (end, disk->dev, &(pos->end), pos->start.sector, opts))
-                return 0;
-
-	return 1;
+  char *start;
+  char *end;
+  if (ped_unit_get_default() == PED_UNIT_CYLINDER) 
+    { 
+      start = _("First cylinder");
+      end = 
+	(compat_mode == 0) ? 
+	    _("Last cilynder or +size or +sizeMB or +sizeKB or +size%")
+	  : _("Last cylinder or +size or +sizeMB or +sizeKB");
+    }
+  else if (ped_unit_get_default() == PED_UNIT_SECTOR) 
+    {
+      start = _("First sector");
+      end = 
+	(compat_mode == 0) ?
+	  _("Last sector or +size or +sizeMB or +sizeKB or +size%")
+	: _("Last sector or +size or +sizeMB or +sizeKB");
+    }
+  else {
+    start = _("Start");
+    end = _("End or +size");
+  }
+  
+  if (!get_sector (start, disk->dev, &(pos->start), -1LL, opts))
+    return 0;
+  
+  /* If we are creating a new partition and want to look where to
+     place its end ... */
+  if (opts & PLACE_END_NEWPART)
+    place_part_end(disk,pos);
+  
+  /* We give the sector from the first query, so that the user can
+     specify size */
+  if (!get_sector (end, disk->dev, &(pos->end), pos->start.sector, opts))
+    return 0;
+  
+  return 1;
 }
 
 
@@ -1100,7 +1158,7 @@ perform_check (PedDisk* disk, PedPartition* part)
                 return 0;
         if (!ped_file_system_check (fs, uiquery->timer)) {
 		ped_file_system_close (fs);
-		return 0;	
+		return 0;
 	}
 
         ped_file_system_close (fs);
@@ -1120,22 +1178,22 @@ perform_cp (PedDisk* dst_disk, PedPartition* dst, UIOpts opts)
         PedFileSystem*          src_fs;
         PedFileSystem*          dst_fs;
         PedFileSystemType*      dst_fs_type;
-	
+
 	if (!dst_disk)
                 goto error;
 
 	if (opts & UI_WARN_COMMIT)
-		if (!ask_boolean_question 
-		(_("WARNING: This writes all data to disk automatically, continue?"))) 
+		if (!ask_boolean_question
+		(_("WARNING: This writes all data to disk automatically, continue?")))
 			return 0;
 	/* It doesn't hurt to commit at that point, right? */
 	if (uiquery->need_commit && !perform_commit(dst_disk,UI_DEFAULT)) goto error;
-        
+
 
         src_disk = dst_disk;
 	if (!get_disk (_("Source device"), &src_disk))
 		goto error_destroy_disk;
-        
+
 
         if (!get_partition (_("Source partition"),src_disk, &src))
                 goto error_destroy_disk;
@@ -1201,22 +1259,26 @@ perform_mklabel (PedDevice* dev, PedDisk** disk, const PedDiskType* type)
         if (!*disk) ped_exception_catch ();
         ped_exception_leave_all ();*/
 
-        if (*disk) {
-                if (!_disk_warn_busy (*disk)) {
-                        ped_disk_destroy (*disk);
-                        goto error;
-                }
+        if (*disk) 
+	  {
+	    if (!_disk_warn_busy (*disk)) 
+	      {
+		/* http://lists.gnu.org/archive/html/bug-fdisk/2008-12/msg00010.html*/
+		/* ped_disk_destroy (*disk); */
+		goto error;
+	      }
                 ped_disk_destroy (*disk);
         }
-	if (!type) {
-        	if (!get_disk_type (_("New disk label type"), &type))
-                	goto error;
-	}
+	if (!type) 
+	  {
+	    if (!get_disk_type (_("New disk label type"), &type))
+	      goto error;
+	  }
 
         *disk = ped_disk_new_fresh (dev, type);
         if (!*disk)
-                goto error;
-	
+	  goto error;
+
         /*if (!ped_disk_commit (*disk))
                 goto error;*/
 	uiquery->need_commit = 1;
@@ -1235,8 +1297,8 @@ perform_mkfs (PedDisk* disk, PedPartition* part, const PedFileSystemType* type,
         PedFileSystem*          fs;
 	PedConstraint*		constraint = NULL;
 	if (opts & UI_WARN_COMMIT) {
-		if (!ask_boolean_question 
-		(_("WARNING: This writes all data to disk automatically, continue?"))) 
+		if (!ask_boolean_question
+		(_("WARNING: This writes all data to disk automatically, continue?")))
 			return 0;
 	}
 
@@ -1315,7 +1377,7 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
         char*                    part_name = NULL;
         char                     *start_usr = NULL, *end_usr = NULL;
         char                     *start_sol = NULL, *end_sol = NULL;
-        
+
         if (!disk)
                 goto error;
 
@@ -1328,7 +1390,7 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
                         goto error;
         }
 
-            
+
         if (part_type == PED_PARTITION_EXTENDED) {
 		if (opts & UI_MAKEPARTFS) {
                 	ped_exception_throw (PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
@@ -1346,8 +1408,8 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
 
 
         if (ped_disk_type_check_feature (disk->type,
-                                         PED_DISK_TYPE_PARTITION_NAME)) 
-                uiquery->getstring(_("Partition name"), &part_name, NULL, NULL, 1); 
+                                         PED_DISK_TYPE_PARTITION_NAME))
+                uiquery->getstring(_("Partition name"), &part_name, NULL, NULL, 1);
 
 
 	if ((opts & UI_CUSTOM_VALUES) &&
@@ -1367,6 +1429,23 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
         if (!part)
                 goto error;
 
+	/* Check if the partition occupying the space before the place we
+	   are trying to put this one, requires a metadata sectors after it,
+	   and if so, increase the range of the constraint */
+
+
+	fuzzify(pos->start.range, disk->dev, NULL, 0,
+		metadata_tail_sectors(
+			disk_get_prev_nmd_partition(
+				disk,
+				ped_disk_get_partition_by_sector(
+					disk,
+					pos->start.sector
+				)
+			)
+		)
+	);
+
         snap_to_boundaries (&part->geom, NULL, disk,
 	                    pos->start.range, pos->end.range);
 
@@ -1388,15 +1467,15 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
         if (!ped_disk_add_partition (disk, part, final_constraint)) {
 		ped_exception_catch();
                 ped_exception_leave_all();
-               
+
                 if (ped_disk_add_partition (disk, part,
                                         ped_constraint_any (disk->dev))) {
                         start_usr = ped_unit_format (disk->dev, pos->start.sector);
                         end_usr   = ped_unit_format (disk->dev, pos->end.sector);
                         start_sol = ped_unit_format (disk->dev, part->geom.start);
                         end_sol   = ped_unit_format (disk->dev, part->geom.end);
-			/* TODO: With the changes from 06/08/06, we might remove 
-			   the "if" and warn the user always if this is out of 
+			/* TODO: With the changes from 06/08/06, we might remove
+			   the "if" and warn the user always if this is out of
 			   range */
 			if (opts & UI_CUSTOM_VALUES)
 				switch (ped_exception_throw (
@@ -1468,22 +1547,22 @@ perform_mkpart (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
                 goto error;
         if (ped_partition_is_flag_available (part, PED_PARTITION_LBA))
                 ped_partition_set_flag (part, PED_PARTITION_LBA, 1);
-        
+
         /*if (!ped_disk_commit (disk))
                 goto error;*/
 	uiquery->need_commit = 1;
-        
+
         /* clean up */
         ped_constraint_destroy (final_constraint);
         ped_constraint_destroy (user_constraint);
         ped_constraint_destroy (dev_constraint);
 
-        
+
         if (pos->start.range != NULL)
                 ped_geometry_destroy (pos->start.range);
         if (pos->end.range != NULL)
                 ped_geometry_destroy (pos->end.range);
-        
+
         if (start_usr != NULL)
                 ped_free (start_usr);
         if (end_usr != NULL)
@@ -1531,13 +1610,13 @@ perform_mkpartfs (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
 
         PedPartition*            part = NULL;
 
-        
+
         if (!disk)
                 return 0;
 
 	if (opts & UI_WARN_COMMIT) {
-		if (!ask_boolean_question 
-		(_("WARNING: This writes all data to disk automatically, continue?"))) 
+		if (!ask_boolean_question
+		(_("WARNING: This writes all data to disk automatically, continue?")))
 			return 0;
 	}
 
@@ -1559,7 +1638,7 @@ perform_mkpartfs (PedDisk* disk, PartPos *pos, PedPartitionType part_type,
 int
 perform_move (PedDisk *disk,  PedPartition* part, PartPos *pos, UIOpts opts)
 {
-       
+
         PedFileSystem*  fs;
         PedFileSystem*  fs_copy;
         PedConstraint*  constraint;
@@ -1568,10 +1647,10 @@ perform_move (PedDisk *disk,  PedPartition* part, PartPos *pos, UIOpts opts)
 
         if (!disk)
                 goto error;
-	
+
 	if (opts & UI_WARN_COMMIT) {
-		if (!ask_boolean_question 
-		(_("WARNING: This writes all data to disk automatically, continue?"))) 
+		if (!ask_boolean_question
+		(_("WARNING: This writes all data to disk automatically, continue?")))
 			return 0;
 	}
 	/* So the best we can do here is to commit */
@@ -1617,7 +1696,7 @@ perform_move (PedDisk *disk,  PedPartition* part, PartPos *pos, UIOpts opts)
                         ped_file_system_get_copy_constraint (fs, disk->dev),
                         constraint_from_start_end (disk->dev, pos->start.range,
                                                    pos->end.range));
-	/* So here we do something worth commiting and dangerous. UI *must* 
+	/* So here we do something worth commiting and dangerous. UI *must*
 	   do something if we fail and need_commit is true... */
 	uiquery->need_commit = 1;
         if (!ped_disk_set_partition_geom (disk, part, constraint,
@@ -1675,7 +1754,7 @@ perform_name (PedDisk* disk, PedPartition* part, char *name)
 	int n;
         if (!disk)
                 goto error;
-	if (!part) 
+	if (!part)
         	if (!get_partition (_("Partition"), disk, &part))
                 	goto error;
 	if (!name) {
@@ -1863,8 +1942,8 @@ perform_rescue (PedDisk* disk, PedSector start, PedSector end, UIOpts opts)
         if (!disk)
                 goto error;
 	if (opts & UI_WARN_COMMIT)
-		if (!ask_boolean_question 
-		(_("WARNING: This writes all data to disk automatically, continue?"))) 
+		if (!ask_boolean_question
+		(_("WARNING: This writes all data to disk automatically, continue?")))
 			return 0;
 	/* Why the hell I use two seperate ifs, instead of one? I wish I knew */
 	if (uiquery->need_commit)
@@ -1915,13 +1994,13 @@ perform_resize (PedDisk *disk, PedPartition *part, PartPos *pos, UIOpts opts)
         if (!disk)
                 goto error;
 
-	if (!part) 
+	if (!part)
         	if (!get_partition (_("Partition"), disk, &part))
                 	goto error;
 
 	if (!(part->type == PED_PARTITION_EXTENDED || (opts & UI_NO_FS_RESIZE)))  {
-		if (opts & UI_WARN_COMMIT && !ask_boolean_question 
-		(_("WARNING: This writes all data to disk automatically, continue?"))) 
+		if (opts & UI_WARN_COMMIT && !ask_boolean_question
+		(_("WARNING: This writes all data to disk automatically, continue?")))
 			return 0;
 		if (uiquery->need_commit)
 			if (!perform_commit(disk,UI_DEFAULT))
@@ -1934,15 +2013,15 @@ perform_resize (PedDisk *disk, PedPartition *part, PartPos *pos, UIOpts opts)
         }
 
  	if (!get_position(pos, disk,opts & UI_CUSTOM_VALUES))
-		goto error; 
+		goto error;
 
 	/* FIXME: This is most likely not needed */
 	//if (!(opts & UI_CUSTOM_VALUES) && (opts & UI_NO_FS_RESIZE)) {
 	if (opts & UI_RESIZE_INEXACT) {
 		if (part->geom.start != pos->start.sector)
-			fuzzify(pos->start.range,disk->dev,NULL);
+			fuzzify(pos->start.range,disk->dev,NULL, 127, 127);
 		if (part->geom.end != pos->end.sector)
-			fuzzify(pos->end.range,disk->dev,NULL);
+			fuzzify(pos->end.range,disk->dev,NULL, 127, 127);
 	}
 	//}
 	if (!ped_geometry_init (&new_geom, disk->dev, pos->start.sector,
@@ -1961,7 +2040,7 @@ perform_resize (PedDisk *disk, PedPartition *part, PartPos *pos, UIOpts opts)
                         goto error_destroy_constraint;
 		if (part->type == PED_PARTITION_EXTENDED)
                 	ped_partition_set_system (part, NULL);
-		
+
 		uiquery->need_commit = 1;
         } else {
 		fs = ped_file_system_open (&part->geom);
@@ -2008,19 +2087,19 @@ error:
 int
 perform_rm (PedDisk* disk, PedPartition* part)
 {
-       
 
-  
+
+
         if (!disk)
                 goto error;
-	if (!part) 
+	if (!part)
         	if (!get_partition (_("Partition"), disk, &part))
                 	goto error;
         if (!_partition_warn_busy (part))
                 goto error;
 	if (!_warn_ext_not_empty (part))
 		goto error;
-		
+
         ped_disk_delete_partition (disk, part);
         uiquery->need_commit = 1;
         return 1;
@@ -2088,12 +2167,12 @@ perform_set (PedDisk *disk, PedPartition *part, PedPartitionFlag flag, UIOpts op
 		if (!get_part_flag (_("Flag to change"), part, &flag))
 			goto error;
         state = (ped_partition_get_flag (part, flag) == 0 ? 1 : 0);
-        
+
         if (!(opts & UI_FLAG_TOGGLE)) {
                 if (!uiquery->getbool (_("New state"), &state))
 		            goto error;
         }
-    
+
         if (!ped_partition_set_flag (part, flag, state))
 	        	goto error;
     	uiquery->need_commit = 1;
@@ -2108,8 +2187,8 @@ int
 perform_commit (PedDisk* disk, UIOpts opts)
 {
 	if (opts & UI_WARN_COMMIT)
-		if (!ask_boolean_question 
-			(_("WARNING: This writes all data to disk, continue?"))) 
+		if (!ask_boolean_question
+			(_("WARNING: This writes all data to disk, continue?")))
 				return 0;
 	if (ped_disk_commit(disk)) {
 		uiquery->need_commit = 0;
@@ -2156,10 +2235,10 @@ _fix_order_add_part (PedDisk *disk, struct DOSGeomList *data) {
 					data->geom.start, data->geom.end);
 
 	range_start = ped_geometry_new (disk->dev,data->geom.start, 1);
-	range_end = ped_geometry_new (disk->dev,data->geom.end, 1); 
+	range_end = ped_geometry_new (disk->dev,data->geom.end, 1);
 	constraint = constraint_from_start_end(disk->dev,
 						range_start,range_end);
-	memcpy(part->disk_specific, data->part_data, DOS_DATA_SIZE); 
+	memcpy(part->disk_specific, data->part_data, DOS_DATA_SIZE);
 	success = ped_disk_add_partition (disk, part, constraint);
 
 	if (constraint) ped_constraint_destroy(constraint);
@@ -2177,7 +2256,7 @@ fix_partition_order (PedDisk *disk)
 		ped_exception_throw(PED_EXCEPTION_NO_FEATURE,
 		                    PED_EXCEPTION_CANCEL,
 		                    _("Fixing partition order on %s partition "
-		                      "label type not implemented."), 
+		                      "label type not implemented."),
 		                    disk->type->name);
 		return 0;
 	}
@@ -2205,15 +2284,15 @@ fix_partition_order (PedDisk *disk)
 				break;
 			}
 		}
-	}	
-	if (i != -1 && j != -1) 
+	}
+	if (i != -1 && j != -1)
 		return 1;
 
 	/* We make sure that there are no mounted partitions */
 	for (i = 1; i <= 4; i++) {
 		walk = ped_disk_get_partition(disk,i);
 		if (walk && !_partition_warn_busy(walk))
-			return 0; 
+			return 0;
 	}
 
 	/* We will issue warning to the user for now */
@@ -2228,8 +2307,8 @@ fix_partition_order (PedDisk *disk)
 	struct DOSGeomList *pdata = NULL;
 	struct DOSGeomList **ptr = &partitions;
 
-	
-	
+
+
 
 	/* We save the partitions */
 	for (walk = ped_disk_next_partition(disk,NULL); walk;
@@ -2262,7 +2341,7 @@ fix_partition_order (PedDisk *disk)
 		_fix_order_add_part(disk, pdata);
 		pdata = pdata->next;
 	}
-	
+
 	/* We clear the saved data */
 	pdata = partitions;
 	while (pdata) {
@@ -2271,7 +2350,7 @@ fix_partition_order (PedDisk *disk)
 		free(pdata);
 		pdata = next;
 	}
-		
+
 	/* We're done */
 	uiquery->need_commit = 1;
 	return 1;
@@ -2282,6 +2361,7 @@ void
 check_partition_consistency (PedPartition *part)
 {
 	int end_cyl,end_head,end_sector;
+	PedSector tail = 0;
 	char *part_chs = NULL;
 	/* TODO: Calculate, instead of parsing the string */
 	part_chs = ped_unit_format_custom(part->disk->dev,part->geom.end,
@@ -2289,8 +2369,12 @@ check_partition_consistency (PedPartition *part)
 	sscanf(part_chs, "%d,%d,%d", &end_cyl, &end_head, &end_sector);
 	ped_free(part_chs);
 
+	/* If, for example, this is a msdos logical partition, it has a
+           tail of one sector, specifying the next partition. */
+	tail = metadata_tail_sectors(part);
+
 	/* We check that the partition ends at the cylinder boundary */
-	if (end_head != part->disk->dev->bios_geom.heads - 1 ||
+	if (end_head != part->disk->dev->bios_geom.heads - 1 - tail ||
 	    end_sector != part->disk->dev->bios_geom.sectors - 1)
 		ped_exception_throw(PED_EXCEPTION_WARNING, PED_EXCEPTION_OK,
 			_("Partition %i does not end on cylinder boundary."),
@@ -2339,7 +2423,7 @@ partition_print_flags (PedPartition* part)
         PedPartitionFlag        flag;
         int                     first_flag;
         const char*             name;
-        char*                   res = ped_malloc(1); 
+        char*                   res = ped_malloc(1);
         void*                   _res = res;
 
         *res = '\0';
@@ -2427,3 +2511,18 @@ print_partition_types() {
 	}
 	exit(0);
 }
+
+
+int count_logical_partition(PedDisk *disk)
+{
+	int count = 0;
+	PedPartition *walk;
+	for (walk = ped_disk_next_partition (disk, NULL); walk;
+	     walk = ped_disk_next_partition (disk, walk)) {
+		if (walk->type & (PED_PARTITION_FREESPACE | PED_PARTITION_METADATA))
+			continue;
+		if (walk->type & PED_PARTITION_LOGICAL)
+			count++;
+	}
+	return count;
+}
